﻿import { LevelData } from "./LevelData";
import { LevelManager } from "./LevelManager";
import GameUI from "./UI/GameUI";
import { HelperUI } from "./UI/HelperUI";

/**所有的排行榜字段 */
export enum FieldType {
    Name,
    PointNumber
}

@Component
export default class GameStart extends Script {

    /** 当脚本被实例后，会在第一帧更新前调用此函数 */
    protected async onStart(): Promise<void> {

        if (SystemUtil.isClient()) {
            LevelManager.instance.init()

            UIService.show(HelperUI);

            // 将头顶昵称设置为233昵称
            const player = Player.localPlayer

            // 当游戏运行在编辑器上，是获取不到233昵称的

            if (SystemUtil.isPIE) {
                player.character.displayName = "玩家名" + player.userId
            } else {
                player.character.displayName = AccountService.getNickName()
            }

        }

        if (SystemUtil.isServer()) {
            // 设置存储环境
            DataStorage.setTemporaryStorage(SystemUtil.isPIE)

            // 数据存储逻辑
            Event.addClientListener("SavePoint", (player: Player, pointNumber: number) => {
                // 使用数据中心存储数据

                // 改变数据
                DataCenterS.getData(player, LevelData).pointNumber = pointNumber

                // 存储数据
                DataCenterS.getData(player, LevelData).save(true)

                LeaderboardModule.setPlayerValue(player, FieldType.PointNumber, pointNumber)
                LeaderboardModule.setPlayerValue(player, FieldType.Name, player.character.displayName)
            })

            // 在服务端设置排行榜的样式
            LeaderboardModule.setStyle("排行榜", true, 10, 5)
            // 设置排行榜的字段（玩家名称，关卡号）
            LeaderboardModule.addField(FieldType.Name, "玩家名")
            LeaderboardModule.addField(FieldType.PointNumber, "关卡号")
            LeaderboardModule.showRankField("排名", null, "未上榜")

            // 当玩家进入游戏，向排行榜添加一条数据
            Player.onPlayerJoin.add((player: Player) => {
                LeaderboardModule.addPlayer(player)

            })


            // 当玩家退出游戏，将数据从排行榜上删除
            Player.onPlayerLeave.add((player: Player) => {
                LeaderboardModule.removePlayer(player)
            })
        }
    }

    /**
     * 周期函数 每帧执行
     * 此函数执行需要将this.useUpdate赋值为true
     * @param dt 当前帧与上一帧的延迟 / 秒
     */
    protected onUpdate(dt: number): void {

    }

    /** 脚本被销毁时最后一帧执行完调用此函数 */
    protected onDestroy(): void {

    }
}