﻿import CheckPointTrigger from "./CheckPointTrigger"
import { LevelData } from "./LevelData"
import GameUI from "./UI/GameUI"

/**
 * 关卡管理器
 */
export class LevelManager{
    // 单例模式
    private static _instacne:LevelManager
    public static get instance():LevelManager{
        if(LevelManager._instacne==null){
            LevelManager._instacne = new LevelManager()
        }
        return LevelManager._instacne
    }

    /**死亡触发器 */
    private _deathTrigger:Trigger

    /**复活位置 */
    private _rebornPosition:Vector = new Vector(10,0,420)

    /**所有的检查点脚本 */
    public checkPointMap:Map<number,CheckPointTrigger> = new Map()

    /**上一个检查点的序号 */
    public lastPointNumber:number = 0

    public async init(){
        this._deathTrigger =  await GameObject.asyncFindGameObjectById("299CDDA6") as Trigger
        this._deathTrigger.onEnter.add((other:GameObject)=>{
            // 当进入的物体是角色类型
            if(other instanceof Character){
                // 让角色死亡
                this.charDeath(other)
            }
        })

        Event.addLocalListener("CheckPoint",(checkPointTrigger:CheckPointTrigger)=>{

            // 当角色进入新关卡的时候，存储新关卡
            if(checkPointTrigger.pointNumber > this.lastPointNumber){
                // 存储数据
                Event.dispatchToServer("SavePoint",checkPointTrigger.pointNumber)
            }

            this._rebornPosition = checkPointTrigger.gameObject.worldTransform.position.clone()

            this.lastPointNumber = checkPointTrigger.pointNumber
        })


        // 等待数据中心准备完毕
        await DataCenterC.ready()

        UIService.show(GameUI,this.checkPointMap.size,0)  

        // 通过客户端数据中心，获取数据
        let pointNumber =  DataCenterC.getData(LevelData).pointNumber
        // 让角色跳关
        this.jumpToPoint(pointNumber)
    }

    /**让角色死亡 */
    public charDeath(char:Character){
        // 开启布娃娃属性
        char.ragdollEnabled = true
        // 播放特效
        EffectService.playAtPosition("27421",char.worldTransform.position)
        // 播放音效
        SoundService.playSound("120841")
        setTimeout(() => {
            // 让角色复活
            this.charReborn(char)
        }, 3000);

        if(char==Player.localPlayer.character){
            Event.dispatchToLocal("Death")
        }
    }

    /**让角色复活 */
    private charReborn(char:Character){
        if(char == Player.localPlayer.character){
            // 将角色的位置改变到复活点
            char.worldTransform.position = this._rebornPosition.clone()
        }

        // 关闭布娃娃属性
        char.ragdollEnabled = false
    }


    /**
     * 跳转到指定关卡
     * @param pointNumber 关卡号
     */
    public jumpToPoint(pointNumber:number = this.lastPointNumber){
        // 实现跳转
        let checkPoint = this.checkPointMap.get(pointNumber)
        if(checkPoint){
            Player.localPlayer.character.worldTransform.position = checkPoint.gameObject.worldTransform.position.clone()
        }
    }
}